import os
import json
import random
import sys
import time
import uuid

import requests

sys.path.append(os.getcwd())
from common.video_item import VideoItem
from common import PiaoQuanPipeline, AliyunLogger
from common.mq import MQ


class Jryzfklspcheduling(object):
    def __init__(self, platform, mode, rule_dict, user_list, env):
        self.platform = platform
        self.mode = mode
        self.rule_dict = rule_dict
        self.user_list = user_list
        self.env = env
        self.download_cnt = 0
        self.mq = MQ(topic_name="topic_crawler_etl_" + self.env)
        self.limit_flag = False

    def get_video_list(self):
        headers = {
            'Host': 'api.zhiqitec.top',
            'Content-Type': 'application/json',
            'Accept-Language': 'zh-cn',
            'Accept': '*/*',
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E217 MicroMessenger/6.8.0(0x16080000) NetType/WIFI Language/en Branch/Br_trunk MiniProgramEnv/Mac',
            'Referer': 'https://servicewechat.com/wxcf72240d227772e5/2/page-frame.html',
            'token': 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjE3MDIzNjkzMDksIm5iZiI6MTcwMjM2OTMwOSwiZXhwIjoxNzAyMzc2NTA5LCJkYXRhIjp7InVzZXJfaWQiOjYzNjEyNTU1fX0.qUi3dRI3JG0KAdR0INGY8kSGJOn21ilbnbxME3G9ffk'
        }
        page_index = 1
        while True:
            time.sleep(random.randint(1, 10))
            try:
                if self.limit_flag:
                    AliyunLogger.logging(
                        code="2000",
                        platform=self.platform,
                        mode=self.mode,
                        env=self.env,
                        message="本轮已经抓取到足够的数据，自动退出\t{}".format(self.download_cnt),
                    )
                    return
                else:
                    payload = {}
                    url = "https://api.zhiqitec.top/index.php?s=mobile/Video/getList&cid=1&page={}&api_version=4&appid=wxcf72240d227772e5&version=1.9.1&env_version=release&scene=1053".format(page_index)
                    response = requests.request("GET", url, headers=headers, data=payload)
                    video_list = response.json()["data"]["list"]
                    if video_list:
                        for index, video_obj in enumerate(video_list, 1):
                            try:
                                if video_obj.get("title"):
                                    AliyunLogger.logging(
                                        code="1001",
                                        platform=self.platform,
                                        mode=self.mode,
                                        env=self.env,
                                        message="扫描到一条视频",
                                        data=video_obj,
                                    )
                                    self.process_video_obj(video_obj)
                            except Exception as e:
                                AliyunLogger.logging(
                                    code="3000",
                                    platform=self.platform,
                                    mode=self.mode,
                                    env=self.env,
                                    data=video_obj,
                                    message="抓取第{}条的时候出现问题， 报错信息是{}".format(index, e),
                                )
                            page_index += 1
                    else:
                        AliyunLogger.logging(
                            code="2000",
                            platform=self.platform,
                            mode=self.mode,
                            env=self.env,
                            message="已经抓完了，自动退出"
                        )
                        return
            except Exception as e:
                AliyunLogger.logging(
                    code="3000",
                    platform=self.platform,
                    mode=self.mode,
                    env=self.env,
                    message="抓取第{}页时候出现错误， 报错信息是{}".format(page_index + 1, e),
                )

    def process_video_obj(self, video_obj):
        trace_id = self.platform + str(uuid.uuid1())
        our_user = random.choice(self.user_list)
        item = VideoItem()
        item.add_video_info("user_id", our_user["uid"])
        item.add_video_info("user_name", our_user["nick_name"])
        item.add_video_info("video_id", video_obj["id"])
        item.add_video_info("video_title", video_obj["title"])
        item.add_video_info("publish_time_stamp", int(time.time()))
        item.add_video_info("video_url", video_obj["video_url"])
        item.add_video_info("cover_url", video_obj["video_cover"])
        item.add_video_info("out_video_id", video_obj["id"])
        item.add_video_info("platform", self.platform)
        item.add_video_info("strategy", self.mode)
        item.add_video_info("session", "{}-{}".format(self.platform, int(time.time())))
        mq_obj = item.produce_item()
        pipeline = PiaoQuanPipeline(
            platform=self.platform,
            mode=self.mode,
            rule_dict=self.rule_dict,
            env=self.env,
            item=mq_obj,
            trace_id=trace_id,
        )
        if pipeline.process_item():
            self.download_cnt += 1
            self.mq.send_msg(mq_obj)
            AliyunLogger.logging(
                code="1002",
                platform=self.platform,
                mode=self.mode,
                env=self.env,
                message="成功发送至 ETL",
                data=mq_obj,
            )
            if self.download_cnt >= int(
                self.rule_dict.get("videos_cnt", {}).get("min", 200)
            ):
                self.limit_flag = True


if __name__ == '__main__':
    S = Jryzfklspcheduling(
        platform="jieriyingzhufukuaile",
        mode="recommend",
        env="dev",
        rule_dict={},
        user_list=[{'nick_name': "Ivring", 'uid': "1997"}, {'nick_name': "paul", 'uid': "1998"}]
    )
    S.get_video_list()