import json
import os
import random
import sys
import time
import uuid

import requests

sys.path.append(os.getcwd())
from common.video_item import VideoItem
from common import PiaoQuanPipeline, AliyunLogger, tunnel_proxies
from common.mq import MQ


class TianXingRecommend(object):
    def __init__(self, platform, mode, rule_dict, user_list, env):
        self.platform = platform
        self.mode = mode
        self.rule_dict = rule_dict
        self.user_list = user_list
        self.env = env
        self.download_cnt = 0
        self.limit_flag = False
        self.mq = MQ(topic_name="topic_crawler_etl_" + self.env)

    def get_video_list(self):
        url = "https://xs.lhxlwl.cn/index.php?s=/api/article/list"
        headers = {
            'Host': 'xs.lhxlwl.cn',
            'xweb_xhr': '1',
            'platform': 'MP-WEIXIN',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 MicroMessenger/6.8.0(0x16080000) NetType/WIFI MiniProgramEnv/Mac MacWechat/WMPF MacWechat/3.8.5(0x13080510)XWEB/1100',
            'storeid': '10011',
            'content-type': 'application/json;charset=UTF-8',
            'accept': '*/*',
            'sec-fetch-site': 'cross-site',
            'sec-fetch-mode': 'cors',
            'sec-fetch-dest': 'empty',
            'referer': 'https://servicewechat.com/wxd2be8fdf5297fac3/5/page-frame.html',
            'accept-language': 'en-US,en;q=0.9'
        }
        page = 1
        while True:
            time.sleep(random.randint(1, 10))
            try:
                if self.limit_flag:
                    message = "本轮已经抓取到足够的数据，自动退出\t{}".format(self.download_cnt)
                    AliyunLogger.logging(
                        code="2000",
                        platform=self.platform,
                        env=self.env,
                        mode=self.mode,
                        message=message
                    )
                    return
                else:
                    payload = {
                        "categoryId": -1,
                        "page": page
                    }
                    response = requests.request("POST", url, headers=headers, data=json.dumps(payload), proxies=tunnel_proxies())
                    video_list = response.json()['data']['list']
                    page += 1
                    if video_list:
                        for index, video_obj in enumerate(video_list, 1):
                            try:
                                AliyunLogger.logging(
                                    code="1001",
                                    platform=self.platform,
                                    mode=self.mode,
                                    env=self.env,
                                    message="扫描到一条视频",
                                    data=video_obj
                                )
                                self.process_video_obj(video_obj)
                            except Exception as e:
                                AliyunLogger.logging(
                                    code="3000",
                                    platform=self.platform,
                                    mode=self.mode,
                                    env=self.env,
                                    message="抓取第{}条的时候出现问题， 报错信息是{}".format(index, e)
                                )
                    else:
                        AliyunLogger.logging(
                            code="2000",
                            platform=self.platform,
                            mode=self.mode,
                            env=self.env,
                            message="视频已经抓完",
                        )
                        return
            except Exception as e:
                AliyunLogger.logging(
                    code="3000",
                    platform=self.platform,
                    mode=self.mode,
                    env=self.env,
                    message="抓取推荐页的时候出现错误， 报错信息是{}".format(e)
                )

    def process_video_obj(self, video_obj):
        trace_id = self.platform + str(uuid.uuid1())
        article_id = video_obj['article_id']
        url = "https://xs.lhxlwl.cn/index.php?s=/api/article/video"
        payload = {"articleId": article_id}
        headers = {
            'Host': 'xs.lhxlwl.cn',
            'xweb_xhr': '1',
            'platform': 'MP-WEIXIN',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 MicroMessenger/6.8.0(0x16080000) NetType/WIFI MiniProgramEnv/Mac MacWechat/WMPF MacWechat/3.8.5(0x13080510)XWEB/1100',
            'storeid': '10011',
            'content-type': 'application/json;charset=UTF-8',
            'accept': '*/*',
            'sec-fetch-site': 'cross-site',
            'sec-fetch-mode': 'cors',
            'sec-fetch-dest': 'empty',
            'referer': 'https://servicewechat.com/wxd2be8fdf5297fac3/5/page-frame.html',
            'accept-language': 'en-US,en;q=0.9'
        }
        response = requests.request("POST", url, headers=headers, data=json.dumps(payload), proxies=tunnel_proxies())
        video_info = response.json()['data']['detail']
        our_user = random.choice(self.user_list)
        item = VideoItem()
        item.add_video_info("user_id", our_user["uid"])
        item.add_video_info("user_name", our_user["nick_name"])
        item.add_video_info("video_id", video_info["article_id"])
        item.add_video_info("video_title", video_info["title"])
        item.add_video_info("video_url", video_info['video'])
        item.add_video_info("cover_url", video_info["share_image_url"])
        item.add_video_info("play_cnt", video_info['show_views'])
        # item.add_video_info("share_cnt", video_obj['shared'])
        item.add_video_info("out_video_id", video_info["article_id"])
        item.add_video_info("platform", self.platform)
        item.add_video_info("strategy", self.mode)
        item.add_video_info("session", "{}-{}".format(self.platform, int(time.time())))
        mq_obj = item.produce_item()
        pipeline = PiaoQuanPipeline(
            platform=self.platform,
            mode=self.mode,
            rule_dict=self.rule_dict,
            env=self.env,
            item=mq_obj,
            trace_id=trace_id,
        )
        if pipeline.process_item():
            self.download_cnt += 1
            self.mq.send_msg(mq_obj)
            AliyunLogger.logging(
                code="1002",
                platform=self.platform,
                mode=self.mode,
                env=self.env,
                message="成功发送至 ETL",
                data=mq_obj,
                trace_id=trace_id
            )
            if self.download_cnt >= int(
                self.rule_dict.get("videos_cnt", {}).get("min", 200)
            ):
                self.limit_flag = True